/***********************************************************************************/
/*                                                                                 */
/*                         DrawEngine Class Declaration                            */
/*                                                                                 */
/***********************************************************************************/

/*
The DrawEngine object reads data about the scene directly from the 
Scene object, and converts this raw data into a displayable image 
through use of OpenGL.
*/


#ifndef DRAWENGINE_H
#define DRAWENGINE_H

#include "SDL.h"
#ifdef WIN32
	#include <windows.h>
#endif 
#include <GL/gl.h>
#include <GL/glu.h>
#include <IL/il.h>
#include <IL/ilut.h>
#include <IL/ilu.h>
#include "scene.h"
#include "list.h"
#include "entity.h"
#include "mouseEntity.h"
#include "math.h"


class DrawEngine
{

public:

	//Constructor.
	DrawEngine(Scene* scenePtr_);
	~DrawEngine();

	//Initialization functions.
	GLvoid initGL();
	ILvoid initIL();
	bool initSDL();
	GLvoid setupProjection();

	//Set functions.
	bool setScene(Scene* scenePtr_);
	
	//Draw functions.
	void draw();
	void drawList(linkedList*, Entity* camera);
	void drawBackdrop();
	void drawEntity(Entity* ent);
	void drawBillboard(float size);
	void drawLoadScreen();

	//Image load function.
	int LoadImage (char* fileName);


private:

	//Pointers to important objects.
	Scene* scenePtr;
	DynamicEntity* playerPtr;
	CameraEntity* cameraPtr; 
	MouseEntity* mousePtr; 
	Borealis* borealisPtr;

	//Texture variables.
	ILuint texid;
	GLuint image;
	int textures[1024][2];
	int textureLoadScreen;

	float farClip;
	float aspect;

};

#endif