/***********************************************************************************/
/*                                                                                 */
/*                          Dynamic Entity Definition                              */
/*                                                                                 */
/***********************************************************************************/

#include <iostream>
#include "dynamicEntity.h"

using namespace std;


/*------------- Constructor & Destructor ---------------*/

DynamicEntity::DynamicEntity(float x, float y) : Entity(x, y)
{
	textureType = TEX_BLOCK;
	collideType = CT_DYNAMIC;

	vel.x = 0;
	vel.y = 0;
	hitVel.x = 0;
	hitVel.y = 0;

	staticHit = 0;
	dynamicHit = 0;
}


DynamicEntity::~DynamicEntity()
{

}


/*------------------ Set Functions --------------------*/


void DynamicEntity::setVelX(float x_)
{
	vel.x = x_;
}

void DynamicEntity::setVelY(float y_)
{
	vel.y = y_;
}

void DynamicEntity::incrVelX(float x_)
{
	vel.x = vel.x+x_;
}

void DynamicEntity::incrVelY(float y_)
{
	vel.y = vel.y+y_;
}

void DynamicEntity::setHitVelX(float x_)
{
	hitVel.x = x_;
}

void DynamicEntity::setHitVelY(float y_)
{
	hitVel.y = y_;
}

void DynamicEntity::setVelocity(Vector vel_)
{
	vel = vel_;
}


/*------------------- Get Functions ----------------------*/

float DynamicEntity::getVelX(void)
{
	return vel.x;
}

float DynamicEntity::getVelY(void)
{
	return vel.y;
}

Vector DynamicEntity::getVelocity(void)
{
	return vel;
}

float DynamicEntity::getHitVelX(void)
{
	return hitVel.x;
}

float DynamicEntity::getHitVelY(void)
{
	return hitVel.y;
}

float DynamicEntity::getSpeed(void)
{
	return vel.getMagnitude();
}



/*------------------ Update Functions --------------------*/


void DynamicEntity::updateEntity()
{
	if(dynamicHit==1) 
	{
		//this ent swapped velocities with another dynEnt on previous frame
		setVelX(getHitVelX());
		setVelY(getHitVelY());
		dynamicHit = 0;
	}

	//updating position according to current velocity.
	setPosX( getPosX() + getVelX() );
	setPosY( getPosY() + getVelY() );

	setVelY( (float)(getVelY() - 0.02) );

	//damping velocity.
	setVelX( getVelX() - (getVelX()/30) );
	setVelY( getVelY() - (getVelY()/30) );

	age++;
}

void DynamicEntity::collideResponse(Vector proj, DynamicEntity* ent)
{

	if(ent!=NULL && (proj.x != 0 || proj.y != 0))
	{
		//Collision has occurred.

		if(ent->getCollideType() == CT_STATIC) //colliding with a static entity...
		{
			if(proj.x != 0)		//collision on x:
			{
				if(staticHit==0)		//delay 1 frame before taking action
					staticHit++;
				else			
				{	
					//project entity, change x velocity.
					setVelX(0);
					incrPosX(-proj.x);
					staticHit = 0;
				}
			}
			else if(proj.y != 0)	//collision on y:
			{
				//project entity, change y velocity.
				incrPosY(-(proj.y));
				setVelY( (float)-0.1*(getVelY()) );
				staticHit = 0;
			}
		}

		else if(ent->getCollideType() == CT_DYNAMIC || ent->getCollideType() == CT_PLAYER) //colliding with a dynamic entity.
		{
			dynamicHit=1; //next time update function is called, vel will be swapped with hitvel.
			if(proj.x != 0) //collision on x:
			{
				incrPosX(-(proj.x));

				setHitVelX( ent->getVelX( ));
				setHitVelY( getVelY( ));
			}
			else if(proj.y != 0)
			{
				incrPosY(-(proj.y));

				setHitVelY( (float)-0.1*(getVelY())+ent->getVelY() );
				setHitVelX( getVelX( ));
			}
		}
	}
}
