/***********************************************************************************/
/*                                                                                 */
/*                         Dynamic Entity Declaration                              */
/*                                                                                 */
/***********************************************************************************/

/*
The dynamic entity class is a descendant of the entity class.
To the entity class it adds capacity for movement: a velocity vector
and functions for dealing with velocity. It also overwrites the update 
and collision functions of the entity class, providing different
functionality when these functions are called.
*/


#ifndef DYNENT_H
#define DYNENT_H
#include "entity.h"
#include "list.h"

class DynamicEntity : public Entity
{
public:
	
	/*----------Constructor and Destructor-----------*/

	DynamicEntity(float posX = 0, float posY = 0);
	~DynamicEntity();


	/*----------Get Functions-----------*/

	Vector getVelocity(void);
	float getVelX(void);
	float getVelY(void);
	float getHitVelX(void);
	float getHitVelY(void);
	float getSpeed(void);

	/*---------Set Functions-----------*/

	void setVelocity(Vector);
	void setVelX(float x_);
	void setVelY(float y_);
	void incrVelX(float x_);
	void incrVelY(float y_);
	void setHitVelX(float x_);
	void setHitVelY(float y_);

	/*------------Update Functions------------*/

	virtual void updateEntity();
	virtual void collideResponse(Vector, DynamicEntity*);


protected:
	/*-------------Attributes-----------*/

	Vector vel; //velocity vector
	Vector hitVel; //velocity after a hit
	bool dynamicHit; //hit flags
	int staticHit;
};

typedef Entity* EntityPtr;


#endif