/***********************************************************************************/
/*                                                                                 */
/*                         Entity Class Implementation                             */
/*                                                                                 */
/***********************************************************************************/

#include <iostream>
#include "entity.h"
#include "math.h"
#include "vector.h"

using namespace std;


/*------------- Constructor -----------------*/

Entity::Entity(float x, float y)
{
	pos.x = x;
	pos.y = y;

	collideType = CT_STATIC;
	textureType = TEX_BLOCK;
	age = 0;
	size = (float)1.0;
	rotate = 0.0;
	life = 1;
	status=0.0f;
	xpermit = 1;

	targetColour.r = glowColourDefault.r=0.3f;
	targetColour.g = glowColourDefault.g=0.5f;
	targetColour.b = glowColourDefault.b=1.0f;
	targetColour.a = glowColourDefault.a=0.3f;

	lineAlphaDefault=0.0;
	setLineAlpha(lineAlphaDefault);

	setGlowColour(glowColourDefault.r, glowColourDefault.g, glowColourDefault.b, glowColourDefault.a);
}

Entity::~Entity()
{

}


/*----------- Get Functions -----------------*/

Vector Entity::getPosition(void)
{
	return pos;
}

float Entity::getPosX(void)
{
	return pos.x;
}

float Entity::getPosY(void)
{
	return pos.y;
}

bool Entity::getXPermit(void)
{
	return xpermit;
}

int Entity::getCollideType(void)
{
	return collideType;
}

int Entity::getTextureType(void)
{
	return textureType;
}

float Entity::getSize(void)
{
	return size;
}


float Entity::getLife(void)
{
	return life;
}

float Entity::getStatus(void)
{
	return status;
}

int Entity::getAge(void)
{
	return age;
}

float Entity::getLineAlpha()
{
	return lineAlpha;
}

float Entity::getGlowColour(char c)
{
	switch(c)
	{
		case 'R':
			return glowColour.r;
		case 'G':
			return glowColour.g;
		case 'B':
			return glowColour.b;
		case 'A':
			return glowColour.a;
		default:
			return 1;
	}
}

/*------------------ Set Functions --------------------*/

void Entity::setPosition(Vector pos_)
{
	pos.x = pos_.x;
	pos.y = pos_.y;
}

void Entity::setPosX(float x_)
{
	pos.x = x_;
}

void Entity::setPosY(float y_)
{
	pos.y = y_;
}

void Entity::setXPermit(bool permit)
{
	xpermit = permit;
}

void Entity::setSize(float size_)
{
	if(size_ < 0.1)
		size = 0.1f;
	else
		size = size_;
}

void Entity::setLife(float life_)
{
	life = life_;
}

void Entity::setStatus(float status_)
{
	status = status_;
}

void Entity::incrPosX(float x_)
{
	pos.x = pos.x+x_;
}

void Entity::incrPosY(float y_)
{
	pos.y = pos.y+y_;
}


void Entity::setLineAlpha(float a)
{
		lineAlpha=a;
}

void Entity::setGlowColour(float r, float g, float b, float a)
{
		glowColour.r=r;
		glowColour.g=g;
		glowColour.b=b;
		glowColour.a=a;
}

void Entity::setDefaultGlowColour(colour c)
{
	targetColour = c;
	glowColourDefault = c;

}

void Entity::setGlowColour(colour c)
{
	glowColour = c;

}


void Entity::setGlowAlpha(float a)
{
		glowColour.a=a;
}

void Entity::resetGlowColour()
{
		glowColour.r=glowColourDefault.r;
		glowColour.g=glowColourDefault.g;
		glowColour.b=glowColourDefault.b;
		glowColour.a=glowColourDefault.a;
}



void Entity::setTextureType(int textureType_)
{
	textureType = textureType_;
}


colour Entity::shiftColour(colour base, colour target, float shift)
{
	if(base.r != target.r)
	{
		base.r += shift*(target.r - base.r);
	}
	if(base.g != target.g)
	{
		base.g += shift*(target.g - base.g);
	}

	if(base.b != target.b)
	{
		base.b += shift*(target.b - base.b);
	}

	if(base.a != target.a)
	{
		base.a += shift*(target.a - base.a);
	}

	return base;
}


/*------------------ Update Functions --------------------*/

void Entity::updateEntity()
{
	age++;

	if(hit>0)
	{
		hit--;
		targetColour.r = 0.5f;
		targetColour.g = 0.6f;
		targetColour.b = 8.0f;
		targetColour.a = 1.0f;
	}
	else
	{
		targetColour.r = glowColourDefault.r;
		targetColour.g = glowColourDefault.g;
		targetColour.b = glowColourDefault.b;
		targetColour.a = glowColourDefault.a;
	}

	if(status==0) //Blocks with status zero are non-displaying.
	{
		setGlowAlpha(0.0);
	}
	else
	{
		if(glowColour.r != targetColour.r)
		{
			glowColour.r += 0.03f*(targetColour.r - glowColour.r);
		}
		if(glowColour.g != targetColour.g)
		{
			glowColour.g += 0.03f*(targetColour.g - glowColour.g);
		}

		if(glowColour.b != targetColour.b)
		{
			glowColour.b += 0.03f*(targetColour.b - glowColour.b);
		}

		if(glowColour.a != targetColour.a)
		{
			glowColour.a += 0.03f*(targetColour.a - glowColour.a);
		}
	}
}

void Entity::collideResponse(Vector proj, Entity* ent)
{
	
	if(ent!=NULL && (proj.x != 0 || proj.y != 0))
	{
		if(ent->getCollideType()==CT_PLAYER)
		{
			if(ent->getStatus() < 4.0f)
			{
				status=1; //Change status to 1 (display status).
				if(hit==0)
					setGlowColour(ent->getGlowColour('R'), ent->getGlowColour('G'), ent->getGlowColour('B'), 1.0);

				hit=3;
			}
			else
				life=0;
		}

		if(ent->getCollideType()==CT_SPARK && ent->getAge() > 5 && getStatus() > 0)
		{
			setGlowColour(ent->getGlowColour('R'), ent->getGlowColour('G'), ent->getGlowColour('B'), 1.0f);
		}

		if(ent->getCollideType() == CT_MOUSE && getStatus() > 0)
		{
			if(hit==0)
				setGlowColour(ent->getGlowColour('R'), ent->getGlowColour('G'), ent->getGlowColour('B'), 1.0);
			hit=3;
		}
	}
}

