/***********************************************************************************/
/*                                                                                 */
/*                         Input Handler Class Definition                          */
/*                                                                                 */
/***********************************************************************************/

/*
The input handler is responsible for catching user input, 
and affecting changes within the Scene in response to this input.
*/

#include "inputHandler.h"

InputHandler::InputHandler(Scene* scenePtr_)
{
	scenePtr = scenePtr_;

	playerPtr = (PlayerEntity*)scenePtr->getPlayerEnt();
	cameraPtr = scenePtr->getCameraEnt();

	mousePtr = scenePtr->getMouseEnt();
	mousePosX = 0;
	mousePosY = 0;
}

bool InputHandler::setScene(Scene* scenePtr_)
{
	scenePtr = scenePtr_;
	return 0;
}

bool InputHandler::handleEvents()
{
		SDL_Event myEvent;
		keystate = SDL_GetKeyState(NULL);
		mousestate = SDL_GetMouseState(&mousePosX, &mousePosY);

		while ( SDL_PollEvent(&myEvent) )  //event loop: process each queued event individually.
		{
			switch(myEvent.type) //if SDL_QUIT event is polled, shut down program.
			{
			case SDL_QUIT:
				return 1;

			case SDL_MOUSEBUTTONDOWN:
				if(myEvent.button.button == SDL_BUTTON_LEFT)
				{
					mousePtr->setClickStatus(1);
				}
				else if(myEvent.button.button == SDL_BUTTON_RIGHT)
				{
				}
				break;

			case SDL_MOUSEBUTTONUP:
				if(myEvent.button.button == SDL_BUTTON_LEFT)
				{
					mousePtr->setClickStatus(0);
				}
				break;
			}
		}

		if ( keystate[SDLK_ESCAPE] )
			return 1;

		updateMouse();

		scenePtr->updateScene();
		return 0;
}


void InputHandler::updateMouse()
{
				GLint viewport[4];
				GLdouble mvmatrix[16], projmatrix[16];
				glGetIntegerv (GL_VIEWPORT, viewport);
				glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
				glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);
				GLint realy;
				GLdouble x, y, z;
				static GLdouble prevX=0, prevY=0;

				realy = scenePtr->getWindowHeight() - mousePosY - 1;

				gluUnProject ((GLdouble) mousePosX, (GLdouble) realy, 0, mvmatrix, projmatrix, viewport, &x, &y, &z);

				mousePtr->setUnwrappedPosX( (float)x+cameraPtr->getPosX() );
				mousePtr->setUnwrappedPosY( (float)y+cameraPtr->getPosY() );

				mousePtr->setVelX( (float)(x - prevX) );
				mousePtr->setVelY( (float)(y - prevY) );
				prevX = x;
				prevY = y;

				mousePtr->setPosX((float)x+cameraPtr->getPosX());
				mousePtr->setPosY((float)y+cameraPtr->getPosY());
				scenePtr->wrapEntPosition(mousePtr);	
}
