/***********************************************************************************/
/*                                                                                 */
/*                             List Class Definition                               */
/*                                                                                 */
/***********************************************************************************/


#include <iostream>
#include "list.h"

using namespace std;



/*------------Constructor-------------*/

linkedList::linkedList()
{
     baseNode=NULL;
	 nodeBuffer=NULL;
}

/*-------------Get functions-----------*/

Entity* linkedList::getHead()
{
	nodeBuffer=baseNode;

	if(baseNode == NULL)
		return NULL;
	else
		return baseNode->ent;
}


Entity* linkedList::getTail()
{
	node *current;
	current = baseNode;

	if( baseNode == NULL)
		return NULL;
	else
	{
		while( current->next != NULL)
			current = current->next;

		nodeBuffer = current;
		return current->ent;
	}
}

Entity* linkedList::getNext()
{

	if( nodeBuffer == NULL)
		return NULL;
	else
	{
		nodeBuffer = nodeBuffer->next;

		if( nodeBuffer == NULL )
			return NULL;

		else
			return nodeBuffer->ent;
	}
}

Entity* linkedList::getCurrent()
{
	if( nodeBuffer == NULL)
		return NULL;
	else
		return nodeBuffer->ent;
}


int linkedList::count()
{
	node *current;
	int c=0;

	for( current=baseNode; current != NULL; current = current->next )
		c++;

	return c;
}




/*-------------Set functions-------------*/


void linkedList::setStart()
{
	nodeBuffer = baseNode;
}

void linkedList::setEnt(Entity *ent_)
{
	node *current,*previous;
	current = baseNode;

	if(ent_ == NULL)
	{
		nodeBuffer = baseNode;
		return;
	}


	if( current->ent == ent_ ) //if entity is at first node, return
	{
		nodeBuffer = baseNode;
		return;
	}

	previous = current;

	while( current!=NULL )
	{
		if( current->ent == ent_ )
		{
			nodeBuffer = current;
			return;
		}

		previous = current;
		current = current->next;
	}
}



/*------------- Adding and Removing Nodes -------------*/


void linkedList::addTail(Entity* ent_)
{
	node *current,*newNode;

	newNode = new node;
	newNode->ent = ent_;
	newNode->next = NULL;

	if( baseNode == NULL )
	{
		baseNode = newNode;
	}
	else
	{
		current = baseNode;

		while( current->next != NULL )
			current = current->next;

		current->next = newNode;

		//B = current; //store previous node(allow backwards traversals)
		nodeBuffer = newNode;
	}


}

void linkedList::addHead(Entity *ent_)
{
	node *newNode;
	newNode = new node;
	newNode->ent = ent_;

	//swap baseNode and new node around
	newNode->next = baseNode;
	baseNode = newNode;

	//B = NULL;
	nodeBuffer = baseNode;
}

void linkedList::insert( int index, Entity *ent_)
{
	node *current, *previous, *newNode;

	previous=NULL;
	current=baseNode;
	newNode = new node;

	for(int i=0; i < (index-1); i++)
	{
		previous = current;
		current = current->next;

		if( current == NULL )
		{
			// List does not contain "index" elements- inserting at tail.
			current = previous;
		}
   }

	newNode->ent = ent_;
	newNode->next = current->next;
	current->next = newNode;

	nodeBuffer = newNode;
}

void linkedList::deleteNode( Entity *ent_ )
{
	node *current,*previous;
	current = baseNode;
	if( current->ent == ent_ )
	{
		baseNode = current->next;
		nodeBuffer = baseNode;
		delete current;
		return;
	}

	previous = current;
	while( current!=NULL )
	{
		if( current->ent == ent_ )
		{
			previous->next = current->next;
			nodeBuffer = previous; 
			delete current->ent;
			delete current;
			return;
		}

		previous = current;
		current = current->next;
	}
}




linkedList::~linkedList()
{
	node *current;
	if( baseNode == NULL )
		return;

	while( baseNode != NULL )
	{
		delete baseNode->ent;
		current = baseNode->next;
		delete baseNode;
		baseNode = current;
	}
}

