/***********************************************************************************/
/*                                                                                 */
/*                                ~~ "OddSpace" ~~                                 */
/*                                                                                 */
/*	As small game by Tomas Woodbridge                                          */
/*	Innovations Project 2008                                                   */
/*                                                                                 */
/***********************************************************************************/

//To compile for Windows: Link with glu32.lib opengl32.lib SDL.lib SDLmain.lib ILUT.lib ILU.lib and DevIL.lib.
//Requires ILU.dll, ILUT.dll and DevIL.dll.


#include "SDL.h"
#ifdef WIN32
	#include <windows.h>
#endif 
#include <GL/gl.h>
#include <GL/glu.h>
#include <IL/il.h>
#include <IL/ilut.h>
#include <IL/ilu.h>
#include <iostream>

#include "list.h"
#include "entity.h"
#include "scene.h"
#include "drawEngine.h"
#include "inputHandler.h"

#define INTERVAL 20

using namespace std;

const GLsizei windowWidth = 800;
const GLsizei windowHeight = 600;

ILvoid initIL();
bool initSDL();
unsigned int timeLeft(void);



/*--------------Main Function----------------*/
 
int main(int argc, char *argv[])
{
	cout << "Loading 'OddSpace'" << endl;
	cout << "Tomas Woodbridge Innovations Project" << endl;
	cout << "NCCA 2008" << endl << endl;

	initSDL();

	Scene mainScene(0, windowWidth, windowHeight);
	DrawEngine drawEngine(&mainScene);
	InputHandler inputHandler(&mainScene);

	int done = 0;

	while( !done )
	{
		drawEngine.draw();
		done = inputHandler.handleEvents();	
		SDL_Delay(timeLeft()); 
	}

	//glDeleteTextures(1, &image);
	SDL_Quit();

	mainScene.~Scene();
	return 1;
}



/*--------------Function Definitions------------------*/

bool initSDL()
{
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
	{
		printf("SDL_Init failed\n");
		return 0;
	}
	if ( SDL_SetVideoMode(windowWidth, windowHeight, 0, SDL_OPENGL) == NULL )
	{
		printf("SDL_SetVideoMode failed\n");
		return 0;
	}

	SDL_WM_SetCaption("OddSpace", NULL);

	return 1;
}

unsigned int timeLeft(void) 
{
	static unsigned int endOfPause = 0;
	unsigned int now;

	now = SDL_GetTicks();
	if(endOfPause <= now)
	{
		endOfPause = now+INTERVAL;
		return 0;
	}
	return (endOfPause - now); 
}