/***********************************************************************************/
/*                                                                                 */
/*                      Mouse Entity Class Definition                              */
/*                                                                                 */
/***********************************************************************************/


#include <iostream>
#include "mouseEntity.h"

using namespace std;


/*------------------ Constructor --------------------*/

MouseEntity::MouseEntity(float x, float y) : DynamicEntity(x, y)
{
	setGlowColour(1.0f, 1.0f, 1.0f, 0.0f);
	setLineAlpha(0.0f);
	textureType = TEX_MOUSE;
	collideType = CT_NOHIT;
	hit =0;
	click=0;
	playerGrab=0;
	playerJump=0;
	cameraEnt = NULL;
	previousPos = getPosition();

}

void MouseEntity::updateEntity()
{
	if(cameraEnt == NULL)
		return;

	/*-------- Update Colour --------*/
	if(playerGrab==0)
		setLineAlpha(0.0f);
	else
		setLineAlpha(0.2f);

	/*-------- Update flags --------*/

	if(playerGrab>1)
		playerGrab--;

	hit=0;
	if(click>0)
		click--;

	previousPos = (unwrappedPos - cameraEnt->getPosition());
}

void MouseEntity::collideResponse(Vector proj, Entity* ent)
{
	if(ent!=NULL && (proj.x != 0 || proj.y != 0) 
	&& ent->getCollideType() != CT_NOHIT && getClickStatus() > 0)
	{
		hit=1;
		if(ent->getCollideType() == CT_PLAYER && ent->getStatus() > 1.9f)
		{
			if(playerGrab==0)
				playerGrab=9;

			collideType=CT_NOHIT;

		}
	}
}


void MouseEntity::setClickStatus(bool set)
{
	if(set==1)
	{
		collideType=CT_MOUSE;
		click=2;
	}
	else
	{
		if(playerGrab>1) 
		{	//recently clicked on player
			playerJump=1;
		}

		collideType=CT_NOHIT;
		playerGrab=0;

	}

}

/*-------------- Accessor Functions ---------------*/


int MouseEntity::getClickStatus()
{
	return click;
}


int MouseEntity::getPlayerGrabStatus()
{
	return playerGrab;
}

int MouseEntity::getPlayerJumpStatus()
{
	return playerJump;
}

void MouseEntity::setPlayerJumpStatus(int status)
{
	playerJump = status;
}



float MouseEntity::getUnwrappedPosX()
{
	return unwrappedPos.x;
}
float MouseEntity::getUnwrappedPosY()
{
	return unwrappedPos.y;
}
Vector MouseEntity::getUnwrappedPos()
{
	return unwrappedPos;
}

void MouseEntity::setUnwrappedPosX(float x_)
{
	unwrappedPos.x=x_;
}
void MouseEntity::setUnwrappedPosY(float y_)
{
	unwrappedPos.y=y_;
}
void MouseEntity::setUnwrappedPos(Vector pos)
{
	unwrappedPos = pos;
}


void MouseEntity::setCameraEnt(CameraEntity* cameraEnt_)
{
	cameraEnt = cameraEnt_;
}

float MouseEntity::getSpeed()
{
	return vel.getMagnitude();
}

