/***********************************************************************************/
/*                                                                                 */
/*                      Mouse Entity Class Declaration                             */
/*                                                                                 */
/***********************************************************************************/

/*
The mouse entity is a special derivative of the entity class, used to track 
the "scene-space" position of the mouse pointer. Its position is updated directly
within the input handler object, in response to user input. 
Mouse 'clicks' are implemented as collision events between the mouse entity and other entities.
*/


#ifndef MOUSEENT_H
#define MOUSEENT_H
#include "dynamicEntity.h"
#include "cameraEntity.h"
#include "Vector.h"

class MouseEntity : public DynamicEntity
{
public:
	MouseEntity(float x, float y);

	void updateEntity();
	void collideResponse(Vector, Entity*);

	/*----------Get functions---------*/

	int getClickStatus();	
	int getPlayerGrabStatus();
	int getPlayerJumpStatus();
	float getUnwrappedPosX();
	float getUnwrappedPosY();
	Vector getUnwrappedPos();
	float getSpeed();

	/*----------Set functions---------*/

	void setClickStatus(bool);
	void setUnwrappedPosX(float);
	void setUnwrappedPosY(float);
	void setUnwrappedPos(Vector);
	void setPlayerJumpStatus(int);
	void setCameraEnt(CameraEntity*);

private:
	CameraEntity* cameraEnt;
	Vector unwrappedPos;
	Vector previousPos;
	bool hit;
	int playerGrab;
	int playerJump;
	int click;
	float speed;
};

#endif