/***********************************************************************************/
/*                                                                                 */
/*                      Player Entity Class Declaration                            */
/*                                                                                 */
/***********************************************************************************/

/*
The player entity is a derivative of dynamic entity, with
a different set of responses for collision events and frame
updates. It also has a set of methods for responding to the 
mouse pointer.
*/

#ifndef PLAYERENT_H
#define PLAYERENT_H
#include "dynamicEntity.h"
#include "mouseEntity.h"
#include "math.h"


class PlayerEntity : public DynamicEntity
{
public:

	/*-------Constructor--------*/

	PlayerEntity(float x = 0, float y = 0);

	/*-------Set Funcitons-------*/

	void setMouseEnt(MouseEntity*);

	/*-------Get Funcitons-------*/

	float getMouseDistance(void);
	float getMouseAngle(void);
	int getSparkStatus(void);


	/*-------Update Funcitons-------*/

	void updateEntity(void);
	void collideResponse(Vector proj, DynamicEntity* ent);
	void setStatus(float status_);
	void jump();

private:
	//Scalar 'speed' value
	float speed;

	//Status flags
	int jumpStatus;
	float bigJumpCount;
	int sparks;
	int stage;

	//Colour values
	colour topSpeedColour;

	//Mouse behaviour variables
	MouseEntity* mouseEnt;
	float mouseAngle;
	float previousMouseAngle;
	float mouseAngleVelocity;

};

#endif