/***********************************************************************************/
/*                                                                                 */
/*                       Spark Entity Class Definition                             */
/*                                                                                 */
/***********************************************************************************/

#include <iostream>
#include "sparkEntity.h"
#include "math.h"

using namespace std;

SparkEntity::SparkEntity(float x, float y) : DynamicEntity(x, y)
{
	collideType = CT_SPARK;
	textureType = TEX_SPARK;

	setLineAlpha(1.0f);
	setGlowColour(	((((float)rand()/(RAND_MAX/* + 1*/)))+0.7f), 
					((((float)rand()/(RAND_MAX/* + 1*/)))+0.7f), 
					((((float)rand()/(RAND_MAX/* + 1*/)))+0.7f), 0.6f);

	setVelY(0.5f*((float)rand()/(RAND_MAX/* + 1*/)) );
	setVelX( 0.3f*(((float)rand()/(RAND_MAX/* + 1*/))-0.5f) );

	age = 0;
}


void SparkEntity::updateEntity()
{
	if(dynamicHit==1) 
	{
		//this ent swapped velocities with another dynEnt on previous frame
		setVelX(getHitVelX());
		setVelY(getHitVelY());
		dynamicHit = 0;
	}

	//update position according to current velocity.
	setPosX( getPosX() + getVelX() );
	setPosY( getPosY() + getVelY() );

	setVelY( (float)(getVelY() - 0.02) );

	//damp velocity.
	setVelX( getVelX() - (getVelX()/60) );
	setVelY( getVelY() - (getVelY()/60) );

	if(getVelY() < -0.8)
		life=0;

	if(getSize()>0.3)
		size-=0.3f;
	else
		setSize(getSize()*(getLife()+0.15f));
	age++;
}

void SparkEntity::collideResponse(Vector proj, DynamicEntity* ent)
{

	if(ent!=NULL && (proj.x != 0 || proj.y != 0))
	{

		if(ent->getCollideType() == CT_DYNAMIC || ent->getCollideType() == CT_PLAYER)
		{

			if(age>5)
			{
				life-=0.45f;
				if(proj.x != 0)		//collision on x
				{
					incrPosX(-proj.x);
					setVelX(ent->getVelX());
					if(proj.x<0)
						incrVelX((float)0.1);
					else
						incrVelX((float)-0.1);
				}
				else if(proj.y != 0)	//collision on y
				{
					incrPosY(-proj.y);
					setVelY(ent->getVelY());
					incrVelX( (float)rand() / (RAND_MAX/* + 1*/) * (0.1f - (-0.1f)) + (-0.1f) );
					if(proj.y<0)
						incrVelY((float)0.3);
				}
			}
			else
				age=0;

		}
		else if(ent->getCollideType() == CT_STATIC && ent->getStatus() > 0) //colliding with a static entity.
		{

			if(age>5)
			{			
				life-=0.45f;
				if(proj.x != 0)		//collision on x
				{
					//project entity, change x velocity.
					incrPosX(-proj.x);
					setVelX(-getVelX());
				}
				else if(proj.y != 0)	//collision on y
				{
					//project entity, change y velocity.
					incrPosY(-proj.y);
					setVelY(-getVelY()*0.5f);
					incrVelX( (float)rand() / (RAND_MAX/* + 1*/) * (0.1f - (-0.1f)) + (-0.1f) );
				}
			}
			else
				age=0;
		}
	}
}
